@@ -47,6 +47,13 @@
                 0,                                      \
                 msg)
 
+#if GLIB_CHECK_VERSION(2, 31, 0)
+#define g_mutex_new() g_new0(GMutex, 1)
+#define g_mutex_free(m) g_free(m)
+#define g_cond_new() g_new0(GCond, 1)
+#define g_cond_free(c) g_free(c)
+#endif
+
 struct _EntangleCameraPrivate {
     GMutex *lock;
     GCond *jobCond;
@@ -76,6 +83,7 @@ struct _EntangleCameraPrivate {
     gboolean hasCapture;
     gboolean hasPreview;
     gboolean hasSettings;
+    gboolean hasViewfinder;
 };
 
 G_DEFINE_TYPE(EntangleCamera, entangle_camera, G_TYPE_OBJECT);
@@ -91,6 +99,7 @@ enum {
     PROP_HAS_CAPTURE,
     PROP_HAS_PREVIEW,
     PROP_HAS_SETTINGS,
+    PROP_HAS_VIEWFINDER,
 };
 
 #define ENTANGLE_CAMERA_ERROR entangle_camera_error_quark ()
@@ -220,6 +229,10 @@ static void entangle_camera_get_property(GObject *object,
             g_value_set_boolean(value, priv->hasSettings);
             break;
 
+        case PROP_HAS_VIEWFINDER:
+            g_value_set_boolean(value, priv->hasViewfinder);
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         }
@@ -264,6 +277,11 @@ static void entangle_camera_set_property(GObject *object,
             ENTANGLE_DEBUG("Set has settings %d", priv->hasSettings);
             break;
 
+        case PROP_HAS_VIEWFINDER:
+            priv->hasViewfinder = g_value_get_boolean(value);
+            ENTANGLE_DEBUG("Set has viewfinder %d", priv->hasViewfinder);
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         }
@@ -494,6 +512,17 @@ static void entangle_camera_class_init(EntangleCameraClass *klass)
                                                          G_PARAM_STATIC_NAME |
                                                          G_PARAM_STATIC_NICK |
                                                          G_PARAM_STATIC_BLURB));
+    g_object_class_install_property(object_class,
+                                    PROP_HAS_VIEWFINDER,
+                                    g_param_spec_boolean("has-viewfinder",
+                                                         "Viewfinder supported",
+                                                         "Whether camera viewfinder configuration is supported",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE |
+                                                         G_PARAM_CONSTRUCT_ONLY |
+                                                         G_PARAM_STATIC_NAME |
+                                                         G_PARAM_STATIC_NICK |
+                                                         G_PARAM_STATIC_BLURB));
     ENTANGLE_DEBUG("install prog done");
 
     g_type_class_add_private(klass, sizeof(EntangleCameraPrivate));
@@ -541,6 +570,21 @@ const char *entangle_camera_get_port(EntangleCamera *cam)
 }
 
 
+#ifdef HAVE_GPHOTO25
+static unsigned int do_entangle_camera_progress_start(GPContext *ctx G_GNUC_UNUSED,
+                                                      float target,
+                                                      const char *msg,
+                                                      void *data)
+{
+    EntangleCamera *cam = data;
+    EntangleCameraPrivate *priv = cam->priv;
+
+    if (priv->progress)
+        entangle_progress_start(priv->progress, target, msg);
+
+    return 0; /* XXX what is this actually useful for ? */
+}
+#else
 static unsigned int do_entangle_camera_progress_start(GPContext *ctx G_GNUC_UNUSED,
                                                       float target,
                                                       const char *format,
@@ -550,11 +594,15 @@ static unsigned int do_entangle_camera_progress_start(GPContext *ctx G_GNUC_UNUS
     EntangleCamera *cam = data;
     EntangleCameraPrivate *priv = cam->priv;
 
-    if (priv->progress)
-        entangle_progress_start(priv->progress, target, format, args);
+    if (priv->progress) {
+        char *str = g_strdup_vprintf(format, args);
+        entangle_progress_start(priv->progress, target, str);
+        g_free(str);
+    }
 
     return 0; /* XXX what is this actually useful for ? */
 }
+#endif
 
 static void do_entangle_camera_progress_update(GPContext *ctx G_GNUC_UNUSED,
                                                unsigned int id G_GNUC_UNUSED,
@@ -587,6 +635,20 @@ static void entangle_camera_reset_last_error(EntangleCamera *cam)
     priv->lastError = NULL;
 }
 
+
+#ifdef HAVE_GPHOTO25
+static void do_entangle_camera_error(GPContext *ctx G_GNUC_UNUSED,
+                                     const char *msg,
+                                     void *data)
+{
+    EntangleCamera *cam = data;
+    EntangleCameraPrivate *priv = cam->priv;
+
+    entangle_camera_reset_last_error(cam);
+    priv->lastError = g_strdup(msg);
+    ENTANGLE_DEBUG("Got error %s", priv->lastError);
+}
+#else
 static void do_entangle_camera_error(GPContext *ctx G_GNUC_UNUSED,
                                      const char *fmt,
                                      va_list args,
@@ -599,7 +661,7 @@ static void do_entangle_camera_error(GPContext *ctx G_GNUC_UNUSED,
     priv->lastError = g_strdup_vprintf(fmt, args);
     ENTANGLE_DEBUG("Got error %s", priv->lastError);
 }
-
+#endif
 
 gboolean entangle_camera_connect(EntangleCamera *cam,
                                  GError **error)
@@ -688,6 +750,7 @@ gboolean entangle_camera_connect(EntangleCamera *cam,
         priv->hasPreview = TRUE;
     if (cap.operations & GP_OPERATION_CONFIG)
         priv->hasSettings = TRUE;
+    priv->hasViewfinder = FALSE;
 
     gp_camera_get_summary(priv->cam, &txt, priv->ctx);
     priv->summary = g_strdup(txt.text);
@@ -805,7 +868,8 @@ gboolean entangle_camera_disconnect(EntangleCamera *cam,
 
     gp_camera_unref(priv->cam);
     priv->cam = NULL;
-
+    priv->hasViewfinder = FALSE;
+    
     priv->hasCapture = priv->hasPreview = priv->hasSettings = FALSE;
 
     ret = TRUE;
@@ -1519,6 +1583,39 @@ gboolean entangle_camera_process_events_finish(EntangleCamera *cam,
 }
 
 
+static CameraWidget *
+entangle_camera_find_widget(EntangleCamera *cam,
+                            const gchar *path)
+{
+    EntangleCameraPrivate *priv = cam->priv;
+    gchar **names = g_strsplit(path, "/", 0);
+    CameraWidget *ret = NULL;
+    CameraWidget *curr = priv->widgets;
+    gsize i;
+
+    for (i = 0 ; names[i] != NULL ; i++) {
+        CameraWidget *tmp;
+
+        if (g_str_equal(names[i], "") ||
+            g_str_equal(names[i], "main"))
+            continue;
+
+        if (gp_widget_get_child_by_name(curr,
+                                        names[i],
+                                        &tmp) != GP_OK)
+            goto cleanup;
+
+        curr = tmp;
+    }
+
+    ret = curr;
+
+ cleanup:
+    g_strfreev(names);
+    return ret;
+}
+
+
 static EntangleControl *do_build_controls(EntangleCamera *cam,
                                           const char *path,
                                           CameraWidget *widget,
@@ -1558,6 +1655,38 @@ static EntangleControl *do_build_controls(EntangleCamera *cam,
     switch (type) {
         /* We treat both window and section as just groups */
     case GP_WIDGET_WINDOW:
+        {
+            EntangleControlGroup *grp;
+            ENTANGLE_DEBUG("Add group %s %d %s", fullpath, id, label);
+            grp = entangle_control_group_new(fullpath, id, label, info, ro);
+            for (int i = 0 ; i < gp_widget_count_children(widget) ; i++) {
+                CameraWidget *child;
+                EntangleControl *subctl;
+                const char *childname;
+                if (gp_widget_get_child(widget, i, &child) != GP_OK ||
+                    gp_widget_get_name(child, &childname) != GP_OK) {
+                    g_object_unref(grp);
+                    goto error;
+                }
+
+                /* Actions are exposed as normal APIs
+                 * And "Other ptp properties" 90% dups
+                 */
+                if (g_str_equal(childname, "actions") ||
+                    g_str_equal(childname, "other"))
+                    continue;
+
+                if (!(subctl = do_build_controls(cam, fullpath, child, error))) {
+                    g_object_unref(grp);
+                    goto error;
+                }
+
+                entangle_control_group_add(grp, subctl);
+            }
+
+            ret = ENTANGLE_CONTROL(grp);
+        } break;
+
     case GP_WIDGET_SECTION:
         {
             EntangleControlGroup *grp;
@@ -1680,6 +1809,10 @@ static gboolean do_load_controls(EntangleCamera *cam,
 
     fullpath = g_strdup_printf("%s/%s", path, name);
     ctrl = g_hash_table_lookup(priv->controlPaths, fullpath);
+    if (!ctrl) {
+        ret = TRUE;
+        goto cleanup;
+    }
     entangle_control_set_readonly(ctrl, ro ? TRUE : FALSE);
 
     switch (type) {
@@ -1783,6 +1916,10 @@ static gboolean do_save_controls(EntangleCamera *cam,
 
     fullpath = g_strdup_printf("%s/%s", path, name);
     ctrl = g_hash_table_lookup(priv->controlPaths, fullpath);
+    if (!ctrl) {
+        ret = TRUE;
+        goto cleanup;
+    }
 
     switch (type) {
         /* We treat both window and section as just groups */
@@ -1888,6 +2025,12 @@ gboolean entangle_camera_load_controls(EntangleCamera *cam,
             priv->controlPaths = NULL;
             goto endjob;
         }
+
+        if (entangle_camera_find_widget(cam, "/main/actions/viewfinder") ||
+            entangle_camera_find_widget(cam, "/main/actions/eosviewfinder"))
+            priv->hasViewfinder = TRUE;
+        else
+            priv->hasViewfinder = FALSE;
     }
 
     ret = do_load_controls(cam, "", priv->widgets, error);
@@ -2069,6 +2212,409 @@ EntangleControlGroup *entangle_camera_get_controls(EntangleCamera *cam, GError *
 }
 
 
+gboolean entangle_camera_set_viewfinder(EntangleCamera *cam,
+                                        gboolean enabled,
+                                        GError **error)
+{
+    g_return_val_if_fail(ENTANGLE_IS_CAMERA(cam), FALSE);
+
+    EntangleCameraPrivate *priv = cam->priv;
+    gboolean ret = FALSE;
+    CameraWidget *widget;
+    CameraWidgetType type;
+    int value;
+    int err;
+
+    g_mutex_lock(priv->lock);
+    entangle_camera_begin_job(cam);
+
+    if (priv->cam == NULL) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Controls not available when camera is disconnected"));
+        goto cleanup;
+    }
+
+    if (priv->widgets == NULL) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Controls not available for this camera"));
+        goto cleanup;
+    }
+
+    widget = entangle_camera_find_widget(cam,
+                                         "/main/actions/viewfinder");
+    if (!widget)
+        widget = entangle_camera_find_widget(cam,
+                                             "/main/actions/eosviewfinder");
+    if (!widget) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Viewfinder control not available with this camera"));
+        goto cleanup;
+    }
+
+    if ((err = gp_widget_get_type(widget,
+                                  &type)) != GP_OK) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Unable to fetch widget type"));
+        goto cleanup;
+    }
+
+    if (type != GP_WIDGET_TOGGLE) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Viewfinder control was not a toggle widget"));
+        goto cleanup;
+    }
+
+    value = enabled ? 1 : 0;
+    if ((err = gp_widget_set_value(widget, &value)) != GP_OK) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Failed to set viewfinder state: %s %d"),
+                    gp_port_result_as_string(err), err);
+        goto cleanup;
+    }
+    if ((err = gp_widget_set_changed(widget, 1)) != GP_OK) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Failed to set manual focus state: %s %d"),
+                    gp_port_result_as_string(err), err);
+        goto cleanup;
+    }
+    
+    if ((err = gp_camera_set_config(priv->cam,
+                                    priv->widgets,
+                                    priv->ctx)) != GP_OK) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Unable to save camera control configuration: %s %d"),
+                    gp_port_result_as_string(err), err);
+        goto cleanup;
+    }
+
+    ret = TRUE;
+
+ cleanup:
+    entangle_camera_end_job(cam);
+    g_mutex_unlock(priv->lock);
+    return ret;
+}
+
+
+static void entangle_camera_set_viewfinder_helper(GSimpleAsyncResult *result,
+                                                  GObject *object,
+                                                  GCancellable *cancellable G_GNUC_UNUSED)
+{
+    GError *error = NULL;
+    gpointer data;
+    gboolean enabled;
+
+    data = g_object_get_data(G_OBJECT(result),
+                             "enabled");
+
+    enabled = GPOINTER_TO_INT(data) == 1 ? TRUE : FALSE;
+
+    if (!entangle_camera_set_viewfinder(ENTANGLE_CAMERA(object),
+                                        enabled,
+                                        &error)) {
+        g_simple_async_result_set_from_error(result, error);
+        g_error_free(error);
+    }
+}
+
+
+void entangle_camera_set_viewfinder_async(EntangleCamera *cam,
+                                          gboolean enabled,
+                                          GCancellable *cancellable,
+                                          GAsyncReadyCallback callback,
+                                          gpointer user_data)
+{
+    g_return_if_fail(ENTANGLE_IS_CAMERA(cam));
+
+    GSimpleAsyncResult *result = g_simple_async_result_new(G_OBJECT(cam),
+                                                           callback,
+                                                           user_data,
+                                                           entangle_camera_set_viewfinder_async);
+
+    g_object_set_data(G_OBJECT(result),
+                      "enabled",
+                      GINT_TO_POINTER(enabled ? 1 : 0));
+
+    g_simple_async_result_run_in_thread(result,
+                                        entangle_camera_set_viewfinder_helper,
+                                        G_PRIORITY_DEFAULT,
+                                        cancellable);
+    g_object_unref(result);
+}
+
+
+gboolean entangle_camera_set_viewfinder_finish(EntangleCamera *cam,
+                                               GAsyncResult *result,
+                                               GError **error)
+{
+    g_return_val_if_fail(ENTANGLE_IS_CAMERA(cam), FALSE);
+
+    return !g_simple_async_result_propagate_error(G_SIMPLE_ASYNC_RESULT(result),
+                                                  error);
+}
+
+
+gboolean entangle_camera_autofocus(EntangleCamera *cam,
+                                   GError **error)
+{
+    g_return_val_if_fail(ENTANGLE_IS_CAMERA(cam), FALSE);
+
+    EntangleCameraPrivate *priv = cam->priv;
+    gboolean ret = FALSE;
+    CameraWidget *widget;
+    CameraWidgetType type;
+    int value;
+    int err;
+
+    g_mutex_lock(priv->lock);
+    entangle_camera_begin_job(cam);
+
+    if (priv->cam == NULL) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Controls not available when camera is disconnected"));
+        goto cleanup;
+    }
+
+    if (priv->widgets == NULL) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Controls not available for this camera"));
+        goto cleanup;
+    }
+
+    widget = entangle_camera_find_widget(cam,
+                                         "/main/actions/autofocusdrive");
+    if (!widget) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Autofocus control not available with this camera"));
+        goto cleanup;
+    }
+
+    if ((err = gp_widget_get_type(widget,
+                                  &type)) != GP_OK) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Unable to fetch widget type"));
+        goto cleanup;
+    }
+
+    if (type != GP_WIDGET_TOGGLE) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Autofocus control was not a toggle widget"));
+        goto cleanup;
+    }
+
+    value = 1;
+    if ((err = gp_widget_set_value(widget, &value)) != GP_OK) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Failed to set autofocus state: %s %d"),
+                    gp_port_result_as_string(err), err);
+        goto cleanup;
+    }
+    if ((err = gp_widget_set_changed(widget, 1)) != GP_OK) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Failed to set manual focus state: %s %d"),
+                    gp_port_result_as_string(err), err);
+        goto cleanup;
+    }
+    
+    if ((err = gp_camera_set_config(priv->cam,
+                                    priv->widgets,
+                                    priv->ctx)) != GP_OK) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Unable to save camera control configuration: %s %d"),
+                    gp_port_result_as_string(err), err);
+        goto cleanup;
+    }
+
+    ret = TRUE;
+
+ cleanup:
+    entangle_camera_end_job(cam);
+    g_mutex_unlock(priv->lock);
+    return ret;
+}
+
+
+static void entangle_camera_autofocus_helper(GSimpleAsyncResult *result,
+                                             GObject *object,
+                                             GCancellable *cancellable G_GNUC_UNUSED)
+{
+    GError *error = NULL;
+
+    if (!entangle_camera_autofocus(ENTANGLE_CAMERA(object),
+                                   &error)) {
+        g_simple_async_result_set_from_error(result, error);
+        g_error_free(error);
+    }
+}
+
+
+void entangle_camera_autofocus_async(EntangleCamera *cam,
+                                     GCancellable *cancellable,
+                                     GAsyncReadyCallback callback,
+                                     gpointer user_data)
+{
+    g_return_if_fail(ENTANGLE_IS_CAMERA(cam));
+
+    GSimpleAsyncResult *result = g_simple_async_result_new(G_OBJECT(cam),
+                                                           callback,
+                                                           user_data,
+                                                           entangle_camera_autofocus_async);
+
+    g_simple_async_result_run_in_thread(result,
+                                        entangle_camera_autofocus_helper,
+                                        G_PRIORITY_DEFAULT,
+                                        cancellable);
+    g_object_unref(result);
+}
+
+
+gboolean entangle_camera_autofocus_finish(EntangleCamera *cam,
+                                          GAsyncResult *result,
+                                          GError **error)
+{
+    g_return_val_if_fail(ENTANGLE_IS_CAMERA(cam), FALSE);
+
+    return !g_simple_async_result_propagate_error(G_SIMPLE_ASYNC_RESULT(result),
+                                                  error);
+}
+
+
+gboolean entangle_camera_manualfocus(EntangleCamera *cam,
+                                     gshort delta,
+                                     GError **error)
+{
+    g_return_val_if_fail(ENTANGLE_IS_CAMERA(cam), FALSE);
+
+    EntangleCameraPrivate *priv = cam->priv;
+    gboolean ret = FALSE;
+    CameraWidget *widget;
+    CameraWidgetType type;
+    float value;
+    int err;
+
+    g_mutex_lock(priv->lock);
+    entangle_camera_begin_job(cam);
+
+    if (priv->cam == NULL) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Controls not available when camera is disconnected"));
+        goto cleanup;
+    }
+
+    if (priv->widgets == NULL) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Controls not available for this camera"));
+        goto cleanup;
+    }
+
+    widget = entangle_camera_find_widget(cam,
+                                         "/main/actions/manualfocusdrive");
+    if (!widget) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Manual focus control not available with this camera"));
+        goto cleanup;
+    }
+
+    if ((err = gp_widget_get_type(widget,
+                                  &type)) != GP_OK) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Unable to fetch widget type"));
+        goto cleanup;
+    }
+
+    if (type != GP_WIDGET_RANGE) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Manual focus control was not a range widget"));
+        goto cleanup;
+    }
+
+    value = delta;
+    if ((err = gp_widget_set_value(widget, &value)) != GP_OK) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Failed to set manual focus state: %s %d"),
+                    gp_port_result_as_string(err), err);
+        goto cleanup;
+    }
+    if ((err = gp_widget_set_changed(widget, 1)) != GP_OK) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Failed to set manual focus state: %s %d"),
+                    gp_port_result_as_string(err), err);
+        goto cleanup;
+    }
+    
+    if ((err = gp_camera_set_config(priv->cam,
+                                    priv->widgets,
+                                    priv->ctx)) != GP_OK) {
+        g_set_error(error, ENTANGLE_CAMERA_ERROR, 0,
+                    _("Unable to save camera control configuration: %s %d"),
+                    gp_port_result_as_string(err), err);
+        goto cleanup;
+    }
+
+    ret = TRUE;
+
+ cleanup:
+    entangle_camera_end_job(cam);
+    g_mutex_unlock(priv->lock);
+    return ret;
+}
+
+
+static void entangle_camera_manualfocus_helper(GSimpleAsyncResult *result,
+                                               GObject *object,
+                                               GCancellable *cancellable G_GNUC_UNUSED)
+{
+    GError *error = NULL;
+    gpointer data;
+
+    data = g_object_get_data(G_OBJECT(result),
+                             "delta");
+
+    if (!entangle_camera_manualfocus(ENTANGLE_CAMERA(object),
+                                     GPOINTER_TO_INT(data),
+                                     &error)) {
+        g_simple_async_result_set_from_error(result, error);
+        g_error_free(error);
+    }
+}
+
+
+void entangle_camera_manualfocus_async(EntangleCamera *cam,
+                                       gshort delta,
+                                       GCancellable *cancellable,
+                                       GAsyncReadyCallback callback,
+                                       gpointer user_data)
+{
+    g_return_if_fail(ENTANGLE_IS_CAMERA(cam));
+
+    GSimpleAsyncResult *result = g_simple_async_result_new(G_OBJECT(cam),
+                                                           callback,
+                                                           user_data,
+                                                           entangle_camera_manualfocus_async);
+
+    g_object_set_data(G_OBJECT(result),
+                      "delta",
+                      GINT_TO_POINTER(delta));
+
+    g_simple_async_result_run_in_thread(result,
+                                        entangle_camera_manualfocus_helper,
+                                        G_PRIORITY_DEFAULT,
+                                        cancellable);
+    g_object_unref(result);
+}
+
+
+gboolean entangle_camera_manualfocus_finish(EntangleCamera *cam,
+                                            GAsyncResult *result,
+                                            GError **error)
+{
+    g_return_val_if_fail(ENTANGLE_IS_CAMERA(cam), FALSE);
+
+    return !g_simple_async_result_propagate_error(G_SIMPLE_ASYNC_RESULT(result),
+                                                  error);
+}
+
+
 gboolean entangle_camera_get_has_capture(EntangleCamera *cam)
 {
     g_return_val_if_fail(ENTANGLE_IS_CAMERA(cam), FALSE);
@@ -2114,6 +2660,21 @@ gboolean entangle_camera_get_has_settings(EntangleCamera *cam)
 }
 
 
+gboolean entangle_camera_get_has_viewfinder(EntangleCamera *cam)
+{
+    g_return_val_if_fail(ENTANGLE_IS_CAMERA(cam), FALSE);
+
+    EntangleCameraPrivate *priv = cam->priv;
+    gboolean ret;
+
+    g_mutex_lock(priv->lock);
+    ret = priv->hasViewfinder;
+    g_mutex_unlock(priv->lock);
+
+    return ret;
+}
+
+
 void entangle_camera_set_progress(EntangleCamera *cam, EntangleProgress *prog)
 {
     g_return_if_fail(ENTANGLE_IS_CAMERA(cam));
